@@ -78,6 +78,7 @@ ZEND_GET_MODULE(geos)
 
 static void noticeHandler(const char *fmt, ...)
 {
+    TSRMLS_FETCH();
     char message[256];
     va_list args;
     va_start(args, fmt);
@@ -89,6 +90,7 @@ static void noticeHandler(const char *fmt, ...)
 
 static void errorHandler(const char *fmt, ...)
 {
+    TSRMLS_FETCH();
     char message[256];
     va_list args;
     va_start(args, fmt);
@@ -96,7 +98,7 @@ static void errorHandler(const char *fmt, ...)
     va_end(args);
 
     /* TODO: use a GEOSException ? */
-    zend_throw_exception_ex(zend_exception_get_default(TSRMLS_CC),
+    zend_throw_exception_ex(zend_exception_get_default(TSRMLS_C),
         1 TSRMLS_CC, "%s", message); 
 
 }
@@ -108,12 +110,14 @@ typedef struct Proxy_t {
 
 static void 
 setRelay(zval* val, void* obj) {
+    TSRMLS_FETCH();
     Proxy* proxy = (Proxy*)zend_object_store_get_object(val TSRMLS_CC);
     proxy->relay = obj;
 }
 
 static inline void *
 getRelay(zval* val, zend_class_entry* ce) {
+    TSRMLS_FETCH();
     Proxy *proxy =  (Proxy*)zend_object_store_get_object(val TSRMLS_CC);
     if ( proxy->std.ce != ce ) {
         php_error_docref(NULL TSRMLS_CC, E_ERROR,
@@ -153,10 +157,10 @@ static long getZvalAsDouble(zval* val)
 }
 
 static zend_object_value
-Gen_create_obj (zend_class_entry *type TSRMLS_DC,
+Gen_create_obj (zend_class_entry *type,
     zend_objects_free_object_storage_t st, zend_object_handlers* handlers)
 {
-    zval *tmp;
+    TSRMLS_FETCH();
     zend_object_value retval;
 
     Proxy *obj = (Proxy *)emalloc(sizeof(Proxy));
@@ -167,7 +171,7 @@ Gen_create_obj (zend_class_entry *type TSRMLS_DC,
     zend_hash_init(obj->std.properties, 0, NULL, ZVAL_PTR_DTOR, 0);
 #if PHP_VERSION_ID < 50399 
     zend_hash_copy(obj->std.properties, &type->default_properties,
-        (copy_ctor_func_t)zval_add_ref, (void *)&tmp, sizeof(zval *));
+        (copy_ctor_func_t)zval_add_ref, NULL, sizeof(zval *));
 #else
     object_properties_init(&(obj->std), type);
 #endif
@@ -199,6 +203,7 @@ PHP_METHOD(Geometry, centroid);
 PHP_METHOD(Geometry, relate); 
 PHP_METHOD(Geometry, relateBoundaryNodeRule); 
 PHP_METHOD(Geometry, simplify); /* also does topology-preserving */
+PHP_METHOD(Geometry, normalize);
 PHP_METHOD(Geometry, extractUniquePoints); 
 PHP_METHOD(Geometry, disjoint);
 PHP_METHOD(Geometry, touches);
@@ -240,6 +245,10 @@ PHP_METHOD(Geometry, length);
 PHP_METHOD(Geometry, distance);
 PHP_METHOD(Geometry, hausdorffDistance);
 PHP_METHOD(Geometry, snapTo);
+PHP_METHOD(Geometry, node);
+PHP_METHOD(Geometry, delaunayTriangulation);
+PHP_METHOD(Geometry, voronoiDiagram);
+PHP_METHOD(Geometry, clipByRect);
 
 static zend_function_entry Geometry_methods[] = {
     PHP_ME(Geometry, __construct, NULL, 0)
@@ -260,6 +269,7 @@ static zend_function_entry Geometry_methods[] = {
     PHP_ME(Geometry, relate, NULL, 0)
     PHP_ME(Geometry, relateBoundaryNodeRule, NULL, 0)
     PHP_ME(Geometry, simplify, NULL, 0)
+    PHP_ME(Geometry, normalize, NULL, 0)
     PHP_ME(Geometry, extractUniquePoints, NULL, 0)
     PHP_ME(Geometry, disjoint, NULL, 0)
     PHP_ME(Geometry, touches, NULL, 0)
@@ -301,6 +311,10 @@ static zend_function_entry Geometry_methods[] = {
     PHP_ME(Geometry, distance, NULL, 0)
     PHP_ME(Geometry, hausdorffDistance, NULL, 0)
     PHP_ME(Geometry, snapTo, NULL, 0)
+    PHP_ME(Geometry, node, NULL, 0)
+    PHP_ME(Geometry, delaunayTriangulation, NULL, 0)
+    PHP_ME(Geometry, voronoiDiagram, NULL, 0)
+    PHP_ME(Geometry, clipByRect, NULL, 0)
     {NULL, NULL, NULL}
 };
 
@@ -407,6 +421,7 @@ Geometry_deserialize(zval **object, zend_class_entry *ce, const unsigned char *b
 static void
 dumpGeometry(GEOSGeometry* g, zval* array)
 {
+    TSRMLS_FETCH();
     int ngeoms, i;
 
     /*
@@ -756,6 +771,30 @@ PHP_METHOD(Geometry, intersection)
     setRelay(return_value, ret);
 }
 
+/**
+ * GEOSGeometry GEOSGeometry::clipByRect(xmin,ymin,xmax,ymax)
+ */
+PHP_METHOD(Geometry, clipByRect)
+{
+    GEOSGeometry *this;
+    GEOSGeometry *ret;
+    double xmin,ymin,xmax,ymax;
+
+    this = (GEOSGeometry*)getRelay(getThis(), Geometry_ce_ptr);
+
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "dddd",
+            &xmin, &ymin, &xmax, &ymax) == FAILURE) {
+        RETURN_NULL();
+    }
+
+    ret = GEOSClipByRect(this, xmin, ymin, xmax, ymax);
+    if ( ! ret ) RETURN_NULL(); /* should get an exception first */
+
+    /* return_value is a zval */
+    object_init_ex(return_value, Geometry_ce_ptr);
+    setRelay(return_value, ret);
+}
+
 PHP_METHOD(Geometry, convexHull)
 {
     GEOSGeometry *this;
@@ -1002,6 +1041,27 @@ PHP_METHOD(Geometry, simplify)
     setRelay(return_value, ret);
 }
 
+/**
+ * GEOSGeometry GEOSGeometry::normalize()
+ */
+PHP_METHOD(Geometry, normalize)
+{
+    GEOSGeometry *this;
+    GEOSGeometry *ret;
+
+    this = (GEOSGeometry*)getRelay(getThis(), Geometry_ce_ptr);
+
+    ret = GEOSGeom_clone(this);
+
+    if ( ! ret ) RETURN_NULL();
+
+    GEOSNormalize(ret); /* exception should be gotten automatically */
+
+    /* return_value is a zval */
+    object_init_ex(return_value, Geometry_ce_ptr);
+    setRelay(return_value, ret);
+}
+
 /**
  * GEOSGeometry GEOSGeometry::extractUniquePoints()
  */
@@ -1907,6 +1967,21 @@ PHP_METHOD(Geometry, snapTo)
     setRelay(return_value, ret);
 }
 
+PHP_METHOD(Geometry, node)
+{
+    GEOSGeometry *this;
+    GEOSGeometry *ret;
+
+    this = (GEOSGeometry*)getRelay(getThis(), Geometry_ce_ptr);
+
+    ret = GEOSNode(this);
+    if ( ! ret ) RETURN_NULL(); /* should get an exception first */
+
+    /* return_value is a zval */
+    object_init_ex(return_value, Geometry_ce_ptr);
+    setRelay(return_value, ret);
+}
+
 
 
 /* -- class GEOSWKTReader -------------------- */
@@ -2162,6 +2237,7 @@ PHP_METHOD(WKBWriter, getByteOrder);
 PHP_METHOD(WKBWriter, setByteOrder);
 PHP_METHOD(WKBWriter, setIncludeSRID);
 PHP_METHOD(WKBWriter, getIncludeSRID);
+PHP_METHOD(WKBWriter, write);
 PHP_METHOD(WKBWriter, writeHEX);
 
 static zend_function_entry WKBWriter_methods[] = {
@@ -2172,6 +2248,7 @@ static zend_function_entry WKBWriter_methods[] = {
     PHP_ME(WKBWriter, setByteOrder, NULL, 0)
     PHP_ME(WKBWriter, getIncludeSRID, NULL, 0)
     PHP_ME(WKBWriter, setIncludeSRID, NULL, 0)
+    PHP_ME(WKBWriter, write, NULL, 0)
     PHP_ME(WKBWriter, writeHEX, NULL, 0)
     {NULL, NULL, NULL}
 };
@@ -2250,6 +2327,38 @@ PHP_METHOD(WKBWriter, setOutputDimension)
 
 }
 
+/**
+ * string GEOSWKBWriter::write(GEOSGeometry)
+ */
+PHP_METHOD(WKBWriter, write)
+{
+    GEOSWKBWriter *writer;
+    zval *zobj;
+    GEOSGeometry *geom;
+    char *ret;
+    size_t retsize;
+    char* retstr;
+
+    writer = (GEOSWKBWriter*)getRelay(getThis(), WKBWriter_ce_ptr);
+
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "o", &zobj)
+        == FAILURE)
+    {
+        RETURN_NULL();
+    }
+
+    geom = getRelay(zobj, Geometry_ce_ptr);
+
+    ret = (char*)GEOSWKBWriter_write(writer, geom, &retsize);
+    /* we'll probably get an exception if ret is null */
+    if ( ! ret ) RETURN_NULL();
+
+    retstr = estrndup(ret, retsize);
+    GEOSFree(ret);
+
+    RETURN_STRINGL(retstr, retsize, 0);
+}
+
 /**
  * string GEOSWKBWriter::writeHEX(GEOSGeometry)
  */
@@ -2358,10 +2467,12 @@ PHP_METHOD(WKBWriter, setIncludeSRID)
 /* -- class GEOSWKBReader -------------------- */
 
 PHP_METHOD(WKBReader, __construct);
+PHP_METHOD(WKBReader, read);
 PHP_METHOD(WKBReader, readHEX);
 
 static zend_function_entry WKBReader_methods[] = {
     PHP_ME(WKBReader, __construct, NULL, 0)
+    PHP_ME(WKBReader, read, NULL, 0)
     PHP_ME(WKBReader, readHEX, NULL, 0)
     {NULL, NULL, NULL}
 };
@@ -2403,6 +2514,31 @@ PHP_METHOD(WKBReader, __construct)
     setRelay(object, obj);
 }
 
+PHP_METHOD(WKBReader, read)
+{
+    GEOSWKBReader *reader;
+    GEOSGeometry *geom;
+    unsigned char* wkb;
+    int wkblen;
+
+    reader = (GEOSWKBReader*)getRelay(getThis(), WKBReader_ce_ptr);
+
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s",
+        &wkb, &wkblen) == FAILURE)
+    {
+        RETURN_NULL();
+    }
+
+    geom = GEOSWKBReader_read(reader, wkb, wkblen);
+    /* we'll probably get an exception if geom is null */
+    if ( ! geom ) RETURN_NULL();
+
+    /* return_value is a zval */
+    object_init_ex(return_value, Geometry_ce_ptr);
+    setRelay(return_value, geom);
+
+}
+
 PHP_METHOD(WKBReader, readHEX)
 {
     GEOSWKBReader *reader;
@@ -2565,6 +2701,78 @@ PHP_FUNCTION(GEOSSharedPaths)
     setRelay(return_value, geom_out);
 }
 
+/**
+ * GEOSGeometry::delaunayTriangulation([<tolerance>], [<onlyEdges>])
+ *
+ *  'tolerance'
+ *       Type: double
+ *       snapping tolerance to use for improved robustness
+ *  'onlyEdges'
+ *       Type: boolean
+ *       if true will return a MULTILINESTRING, otherwise (the default)
+ *       it will return a GEOMETRYCOLLECTION containing triangular POLYGONs.
+ */
+PHP_METHOD(Geometry, delaunayTriangulation)
+{
+    GEOSGeometry *this;
+    GEOSGeometry *ret;
+    double tolerance = 0.0;
+    zend_bool edgeonly = 0;
+
+    this = (GEOSGeometry*)getRelay(getThis(), Geometry_ce_ptr);
+
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|db",
+            &tolerance, &edgeonly) == FAILURE) {
+        RETURN_NULL();
+    }
+
+    ret = GEOSDelaunayTriangulation(this, tolerance, edgeonly ? 1 : 0);
+    if ( ! ret ) RETURN_NULL(); /* should get an exception first */
+
+    /* return_value is a zval */
+    object_init_ex(return_value, Geometry_ce_ptr);
+    setRelay(return_value, ret);
+}
+
+/**
+ * GEOSGeometry::voronoiDiagram([<tolerance>], [<onlyEdges>], [<extent>])
+ *
+ *  'tolerance'
+ *       Type: double
+ *       snapping tolerance to use for improved robustness
+ *  'onlyEdges'
+ *       Type: boolean
+ *       if true will return a MULTILINESTRING, otherwise (the default)
+ *       it will return a GEOMETRYCOLLECTION containing POLYGONs.
+ *  'extent'
+ *       Type: geometry
+ *       Clip returned diagram by the extent of the given geometry
+ */
+PHP_METHOD(Geometry, voronoiDiagram)
+{
+    GEOSGeometry *this;
+    GEOSGeometry *ret;
+    zval *zobj = 0;
+    GEOSGeometry *env = 0;
+    double tolerance = 0.0;
+    zend_bool edgeonly = 0;
+
+    this = (GEOSGeometry*)getRelay(getThis(), Geometry_ce_ptr);
+
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|dbo",
+            &tolerance, &edgeonly, &zobj) == FAILURE) {
+        RETURN_NULL();
+    }
+
+    if ( zobj ) env = getRelay(zobj, Geometry_ce_ptr);
+    ret = GEOSVoronoiDiagram(this, env, tolerance, edgeonly ? 1 : 0);
+    if ( ! ret ) RETURN_NULL(); /* should get an exception first */
+
+    /* return_value is a zval */
+    object_init_ex(return_value, Geometry_ce_ptr);
+    setRelay(return_value, ret);
+}
+
 /**
  * bool GEOSRelateMatch(string matrix, string pattern)
  */
