@@ -8,6 +8,57 @@ dl("geos.so");
 if ( version_compare(PHPUnit_Runner_Version::id(), '3.6') < 0 )
     require_once 'PHPUnit/Framework.php';
 
+# hex2bin for PHP < 5.4
+# https://gist.github.com/mcrumley/5672621
+if (!function_exists('hex2bin')) {
+  function hex2bin($str) {
+    $map = array(
+      '00'=>"\x00", '10'=>"\x10", '20'=>"\x20", '30'=>"\x30", '40'=>"\x40", '50'=>"\x50", '60'=>"\x60", '70'=>"\x70",
+      '01'=>"\x01", '11'=>"\x11", '21'=>"\x21", '31'=>"\x31", '41'=>"\x41", '51'=>"\x51", '61'=>"\x61", '71'=>"\x71",
+      '02'=>"\x02", '12'=>"\x12", '22'=>"\x22", '32'=>"\x32", '42'=>"\x42", '52'=>"\x52", '62'=>"\x62", '72'=>"\x72",
+      '03'=>"\x03", '13'=>"\x13", '23'=>"\x23", '33'=>"\x33", '43'=>"\x43", '53'=>"\x53", '63'=>"\x63", '73'=>"\x73",
+      '04'=>"\x04", '14'=>"\x14", '24'=>"\x24", '34'=>"\x34", '44'=>"\x44", '54'=>"\x54", '64'=>"\x64", '74'=>"\x74",
+      '05'=>"\x05", '15'=>"\x15", '25'=>"\x25", '35'=>"\x35", '45'=>"\x45", '55'=>"\x55", '65'=>"\x65", '75'=>"\x75",
+      '06'=>"\x06", '16'=>"\x16", '26'=>"\x26", '36'=>"\x36", '46'=>"\x46", '56'=>"\x56", '66'=>"\x66", '76'=>"\x76",
+      '07'=>"\x07", '17'=>"\x17", '27'=>"\x27", '37'=>"\x37", '47'=>"\x47", '57'=>"\x57", '67'=>"\x67", '77'=>"\x77",
+      '08'=>"\x08", '18'=>"\x18", '28'=>"\x28", '38'=>"\x38", '48'=>"\x48", '58'=>"\x58", '68'=>"\x68", '78'=>"\x78",
+      '09'=>"\x09", '19'=>"\x19", '29'=>"\x29", '39'=>"\x39", '49'=>"\x49", '59'=>"\x59", '69'=>"\x69", '79'=>"\x79",
+      '0a'=>"\x0a", '1a'=>"\x1a", '2a'=>"\x2a", '3a'=>"\x3a", '4a'=>"\x4a", '5a'=>"\x5a", '6a'=>"\x6a", '7a'=>"\x7a",
+      '0b'=>"\x0b", '1b'=>"\x1b", '2b'=>"\x2b", '3b'=>"\x3b", '4b'=>"\x4b", '5b'=>"\x5b", '6b'=>"\x6b", '7b'=>"\x7b",
+      '0c'=>"\x0c", '1c'=>"\x1c", '2c'=>"\x2c", '3c'=>"\x3c", '4c'=>"\x4c", '5c'=>"\x5c", '6c'=>"\x6c", '7c'=>"\x7c",
+      '0d'=>"\x0d", '1d'=>"\x1d", '2d'=>"\x2d", '3d'=>"\x3d", '4d'=>"\x4d", '5d'=>"\x5d", '6d'=>"\x6d", '7d'=>"\x7d",
+      '0e'=>"\x0e", '1e'=>"\x1e", '2e'=>"\x2e", '3e'=>"\x3e", '4e'=>"\x4e", '5e'=>"\x5e", '6e'=>"\x6e", '7e'=>"\x7e",
+      '0f'=>"\x0f", '1f'=>"\x1f", '2f'=>"\x2f", '3f'=>"\x3f", '4f'=>"\x4f", '5f'=>"\x5f", '6f'=>"\x6f", '7f'=>"\x7f",
+
+      '80'=>"\x80", '90'=>"\x90", 'a0'=>"\xa0", 'b0'=>"\xb0", 'c0'=>"\xc0", 'd0'=>"\xd0", 'e0'=>"\xe0", 'f0'=>"\xf0",
+      '81'=>"\x81", '91'=>"\x91", 'a1'=>"\xa1", 'b1'=>"\xb1", 'c1'=>"\xc1", 'd1'=>"\xd1", 'e1'=>"\xe1", 'f1'=>"\xf1",
+      '82'=>"\x82", '92'=>"\x92", 'a2'=>"\xa2", 'b2'=>"\xb2", 'c2'=>"\xc2", 'd2'=>"\xd2", 'e2'=>"\xe2", 'f2'=>"\xf2",
+      '83'=>"\x83", '93'=>"\x93", 'a3'=>"\xa3", 'b3'=>"\xb3", 'c3'=>"\xc3", 'd3'=>"\xd3", 'e3'=>"\xe3", 'f3'=>"\xf3",
+      '84'=>"\x84", '94'=>"\x94", 'a4'=>"\xa4", 'b4'=>"\xb4", 'c4'=>"\xc4", 'd4'=>"\xd4", 'e4'=>"\xe4", 'f4'=>"\xf4",
+      '85'=>"\x85", '95'=>"\x95", 'a5'=>"\xa5", 'b5'=>"\xb5", 'c5'=>"\xc5", 'd5'=>"\xd5", 'e5'=>"\xe5", 'f5'=>"\xf5",
+      '86'=>"\x86", '96'=>"\x96", 'a6'=>"\xa6", 'b6'=>"\xb6", 'c6'=>"\xc6", 'd6'=>"\xd6", 'e6'=>"\xe6", 'f6'=>"\xf6",
+      '87'=>"\x87", '97'=>"\x97", 'a7'=>"\xa7", 'b7'=>"\xb7", 'c7'=>"\xc7", 'd7'=>"\xd7", 'e7'=>"\xe7", 'f7'=>"\xf7",
+      '88'=>"\x88", '98'=>"\x98", 'a8'=>"\xa8", 'b8'=>"\xb8", 'c8'=>"\xc8", 'd8'=>"\xd8", 'e8'=>"\xe8", 'f8'=>"\xf8",
+      '89'=>"\x89", '99'=>"\x99", 'a9'=>"\xa9", 'b9'=>"\xb9", 'c9'=>"\xc9", 'd9'=>"\xd9", 'e9'=>"\xe9", 'f9'=>"\xf9",
+      '8a'=>"\x8a", '9a'=>"\x9a", 'aa'=>"\xaa", 'ba'=>"\xba", 'ca'=>"\xca", 'da'=>"\xda", 'ea'=>"\xea", 'fa'=>"\xfa",
+      '8b'=>"\x8b", '9b'=>"\x9b", 'ab'=>"\xab", 'bb'=>"\xbb", 'cb'=>"\xcb", 'db'=>"\xdb", 'eb'=>"\xeb", 'fb'=>"\xfb",
+      '8c'=>"\x8c", '9c'=>"\x9c", 'ac'=>"\xac", 'bc'=>"\xbc", 'cc'=>"\xcc", 'dc'=>"\xdc", 'ec'=>"\xec", 'fc'=>"\xfc",
+      '8d'=>"\x8d", '9d'=>"\x9d", 'ad'=>"\xad", 'bd'=>"\xbd", 'cd'=>"\xcd", 'dd'=>"\xdd", 'ed'=>"\xed", 'fd'=>"\xfd",
+      '8e'=>"\x8e", '9e'=>"\x9e", 'ae'=>"\xae", 'be'=>"\xbe", 'ce'=>"\xce", 'de'=>"\xde", 'ee'=>"\xee", 'fe'=>"\xfe",
+      '8f'=>"\x8f", '9f'=>"\x9f", 'af'=>"\xaf", 'bf'=>"\xbf", 'cf'=>"\xcf", 'df'=>"\xdf", 'ef'=>"\xef", 'ff'=>"\xff",
+    );
+    $strlen = strlen($str);
+    if ($strlen % 2 !== 0) {
+      user_error('Hexadecimal input string must have an even length', E_USER_WARNING);
+      return false;
+    }
+    if (strspn($str, '0123456789ABCDEFabcdef') !== $strlen) {
+      return false;
+    }
+    return strtr(strtolower($str), $map);
+  }
+}
+
 
 class test extends PHPUnit_Framework_TestCase
 {
@@ -689,9 +740,9 @@ class test extends PHPUnit_Framework_TestCase
         $g = $reader->read('POLYGON((0 0, 10 0, 10 10, 0 10, 0 0))');
         $g2 = $reader->read('POLYGON((5 -5, 5 5, 15 5, 15 -5, 5 -5))');
         $gi = $g->intersection($g2);
-        $this->assertEquals(
+        $this->assertTrue($gi->equals($reader->read(
 'POLYGON ((10 5, 10 0, 5 0, 5 5, 10 5))'
-            , $writer->write($gi));
+            )));
         $g2 = $reader->read('POLYGON((10 0, 20 0, 20 -5, 10 -5, 10 0))');
         $gi = $g->intersection($g2);
         $this->assertEquals( 'POINT (10 0)'
@@ -702,6 +753,38 @@ class test extends PHPUnit_Framework_TestCase
             , $writer->write($gi));
     }
 
+    public function testGeometry_clipByRect()
+    {
+        $reader = new GEOSWKTReader();
+        $writer = new GEOSWKTWriter();
+        $writer->setRoundingPrecision(0);
+
+        /* POINT */
+        $g = $reader->read('POINT(0 0)');
+        $gi = $g->clipByRect(-1,-1,1,1);
+        $this->assertEquals( 'POINT (0 0)'
+            , $writer->write($gi));
+        $gi = $g->clipByRect(1,1,2,2);
+        $this->assertEquals( 'GEOMETRYCOLLECTION EMPTY'
+            , $writer->write($gi));
+
+        /* LINE */
+        $g = $reader->read('LINESTRING(0 0, 10 0)');
+        $gi = $g->clipByRect(1,-1,2,1);
+        $this->assertEquals( 'LINESTRING (1 0, 2 0)'
+            , $writer->write($gi));
+
+        /* POLY */
+        $g = $reader->read('POLYGON((0 0, 10 0, 10 10, 0 10, 0 0))');
+        $gi = $g->clipByRect(1,1,5,5);
+        $this->assertEquals( 'POLYGON ((1 1, 1 5, 5 5, 5 1, 1 1))'
+            , $writer->write($gi));
+        $gi = $g->clipByRect(-1,-1,5,5);
+        $this->assertEquals( 'POLYGON ((0 0, 0 5, 5 5, 5 0, 0 0))'
+            , $writer->write($gi));
+
+    }
+
     public function testGeometry_convexHull()
     {
         $reader = new GEOSWKTReader();
@@ -1056,7 +1139,7 @@ MULTIPOINT(
         $g = $reader->read('POLYGON((0 0, 0 10, 5 5, 10 10, 10 0, 0 0))');
         $b = $g->pointOnSurface();
         $this->assertEquals(
-'POINT (2 5)'
+'POINT (1 8)'
             , $writer->write($b));
     }
 
@@ -2017,6 +2100,51 @@ MULTIPOINT(
 
     }
 
+    public function testGeometry_delaunayTriangulation()
+    {
+        $reader = new GEOSWKTReader();
+        $writer = new GEOSWKTWriter();
+        $writer->setRoundingPrecision(0);
+
+        $g = $reader->read('POLYGON ((0 0, 1 0, 1 1, 0 1, 0 0))');
+
+        $b = $g->delaunayTriangulation();
+        $this->assertEquals(
+'GEOMETRYCOLLECTION (POLYGON ((0 1, 0 0, 1 0, 0 1)), POLYGON ((0 1, 1 0, 1 1, 0 1)))'
+            , $writer->write($b));
+
+        $b = $g->delaunayTriangulation(0,true);
+        $this->assertEquals(
+'MULTILINESTRING ((0 1, 1 1), (0 0, 0 1), (0 0, 1 0), (1 0, 1 1), (0 1, 1 0))'
+            , $writer->write($b));
+
+    }
+
+    public function testGeometry_voronoiDiagram()
+    {
+        $reader = new GEOSWKTReader();
+        $writer = new GEOSWKTWriter();
+        $writer->setRoundingPrecision(0);
+
+        $g = $reader->read('MULTIPOINT(0 0, 100 0, 100 100, 0 100)');
+
+        $b = $g->voronoiDiagram();
+        $this->assertEquals(
+'GEOMETRYCOLLECTION (POLYGON ((50 50, 50 200, 200 200, 200 50, 50 50)), POLYGON ((50 -100, 50 50, 200 50, 200 -100, 50 -100)), POLYGON ((-100 50, -100 200, 50 200, 50 50, -100 50)), POLYGON ((-100 -100, -100 50, 50 50, 50 -100, -100 -100)))'
+            , $writer->write($b->normalize()));
+
+        $b = $g->voronoiDiagram(0, 1);
+        $this->assertEquals(
+'MULTILINESTRING ((50 50, 200 50), (50 50, 50 200), (50 -100, 50 50), (-100 50, 50 50))'
+            , $writer->write($b->normalize()));
+
+        $b = $g->voronoiDiagram(0, 1, $g->buffer(1000));
+        $this->assertEquals(
+'MULTILINESTRING ((50 50, 1100 50), (50 50, 50 1100), (50 -1000, 50 50), (-1000 50, 50 50))'
+            , $writer->write($b->normalize()));
+
+    }
+
     public function testGeometry_snapTo()
     {
         $reader = new GEOSWKTReader();
@@ -2036,6 +2164,20 @@ MULTIPOINT(
             , $writer->write($snapped) );
     }
 
+    public function testGeometry_node()
+    {
+        $reader = new GEOSWKTReader();
+        $writer = new GEOSWKTWriter();
+        $writer->setTrim(true);
+
+        $g = $reader->read('LINESTRING(0 0, 10 0, 5 -5, 5 5)');
+
+        $noded = $g->node();
+        $this->assertEquals('MULTILINESTRING ((0 0, 5 0), (5 0, 10 0, 5 -5, 5 0), (5 0, 5 5))'
+            , $writer->write($noded) );
+
+    }
+
     public function testWKBWriter__construct()
     {
         $writer = new GEOSWKBWriter();
@@ -2092,7 +2234,6 @@ MULTIPOINT(
         } catch (Exception $e) {
             $this->assertContains('LITTLE (1) or BIG (0)', $e->getMessage());
         }
-
     }
 
     public function testWKBWriter_getsetIncludeSRID()
@@ -2106,117 +2247,79 @@ MULTIPOINT(
         $this->assertEquals(FALSE, $writer->getIncludeSRID());
     }
 
-    public function testWKBWriter_writeHEX()
+    /**
+     * @dataProvider providerWKBWriter_write
+     *
+     * @param integer $byteOrder       The byte order: 0 for BIG endian, 1 for LITTLE endian.
+     * @param integer $inputDimension  The input dimension: 2 or 3.
+     * @param integer $outputDimension The output dimension: 2 or 3.
+     * @param boolean $includeSrid     Whether to include the SRID in the output.
+     * @param string  $wkb             The expected HEX WKB output.
+     */
+    public function testWKBWriter_write($byteOrder, $inputDimension, $outputDimension, $includeSrid, $wkb)
     {
-        $writer = new GEOSWKBWriter();
         $reader = new GEOSWKTReader();
+        $writer = new GEOSWKBWriter();
 
-        try {
-            $writer->writeHEX(1);
-            $this->assertTrue(FALSE); # this is just to fail if we get here
-        } catch (Exception $e) {
-            $this->assertContains('expects parameter 1', $e->getMessage());
+        $writer->setByteOrder($byteOrder);
+        $writer->setOutputDimension($outputDimension);
+        $writer->setIncludeSRID($includeSrid);
+
+        if ($inputDimension === 3) {
+            $g = $reader->read('POINT(6 7 8)');
+            $g->setSRID(53);
+        } else {
+            $g = $reader->read('POINT(6 7)');
+            $g->setSRID(43);
         }
 
-        $g = $reader->read('POINT(6 7)');
-        $g->setSRID(43);
+        $this->assertSame(hex2bin($wkb), $writer->write($g));
+        $this->assertSame($wkb, $writer->writeHEX($g));
+    }
 
-        $writer->setOutputDimension(2); // 2D
+    public function providerWKBWriter_write()
+    {
+        return array(
+            // 2D input
+            array(1, 2, 2, false, '010100000000000000000018400000000000001C40'),        // 2D LITTLE endian
+            array(1, 2, 2, true, '01010000202B00000000000000000018400000000000001C40'), // 2D LITTLE endian + SRID
+            array(0, 2, 2, false, '00000000014018000000000000401C000000000000'),        // 2D BIG endian
+            array(0, 2, 2, true, '00200000010000002B4018000000000000401C000000000000'), // 2D BIG endian + SRID
+            array(1, 2, 3, false, '010100000000000000000018400000000000001C40'),        // 3D LITTLE endian
+            array(1, 2, 3, true, '01010000202B00000000000000000018400000000000001C40'), // 3D LITTLE endian + SRID
+            array(0, 2, 3, false, '00000000014018000000000000401C000000000000'),        // 3D BIG endian
+            array(0, 2, 3, true, '00200000010000002B4018000000000000401C000000000000'), // 3D BIG endian + SRID
 
-        // 2D LITTLE endian
-        $writer->setByteOrder(1);
-        $this->assertEquals(
-            '010100000000000000000018400000000000001C40',
-            $writer->writeHEX($g));
-        // 2D LITTLE endian + SRID 
-        $writer->setIncludeSRID(TRUE);
-        $this->assertEquals(
-            '01010000202B00000000000000000018400000000000001C40',
-            $writer->writeHEX($g));
-        $writer->setIncludeSRID(FALSE);
+            // 3D input
+            array(1, 3, 2, false, '010100000000000000000018400000000000001C40'),                        // 2D LITTLE endian
+            array(1, 3, 2, true, '01010000203500000000000000000018400000000000001C40'),                 // 2D LITTLE endian + SRID
+            array(0, 3, 2, false, '00000000014018000000000000401C000000000000'),                        // 2D BIG endian
+            array(0, 3, 2, true, '0020000001000000354018000000000000401C000000000000'),                 // 2D BIG endian + SRID
+            array(1, 3, 3, false, '010100008000000000000018400000000000001C400000000000002040'),        // 3D LITTLE endian
+            array(1, 3, 3, true, '01010000A03500000000000000000018400000000000001C400000000000002040'), // 3D LITTLE endian + SRID
+            array(0, 3, 3, false, '00800000014018000000000000401C0000000000004020000000000000'),        // 3D BIG endian
+            array(0, 3, 3, true, '00A0000001000000354018000000000000401C0000000000004020000000000000'), // 3D BIG endian + SRID
+        );
+    }
 
-        // 2D BIG endian
-        $writer->setByteOrder(0);
-        $this->assertEquals('00000000014018000000000000401C000000000000',
-            $writer->writeHEX($g));
-        // 2D BIG endian + SRID
-        $writer->setIncludeSRID(TRUE); 
-        $this->assertEquals(
-            '00200000010000002B4018000000000000401C000000000000',
-            $writer->writeHEX($g));
-        $writer->setIncludeSRID(FALSE); 
+    /**
+     * @expectedException \Exception
+     * @expectedExceptionMessage expects parameter 1
+     */
+    public function testInvalidWriteThrowsException()
+    {
+        $writer = new GEOSWKBWriter();
+        $writer->write(1);
+    }
 
-        $writer->setOutputDimension(3); // 3D
-
-        // 3D LITTLE endian (2D input)
-        $writer->setByteOrder(1);
-        $this->assertEquals(
-            '010100000000000000000018400000000000001C40',
-            $writer->writeHEX($g));
-        // 3D LITTLE endian + SRID  (2D input)
-        $writer->setIncludeSRID(TRUE);
-        $this->assertEquals(
-            '01010000202B00000000000000000018400000000000001C40',
-            $writer->writeHEX($g));
-        $writer->setIncludeSRID(FALSE);
-
-        // 3D BIG endian (2D input)
-        $writer->setByteOrder(0);
-        $this->assertEquals('00000000014018000000000000401C000000000000',
-            $writer->writeHEX($g));
-        // 3D BIG endian + SRID (2D input)
-        $writer->setIncludeSRID(TRUE); 
-        $this->assertEquals(
-            '00200000010000002B4018000000000000401C000000000000',
-            $writer->writeHEX($g));
-        $writer->setIncludeSRID(FALSE); 
-
-
-        $g = $reader->read('POINT(6 7 8)');
-        $g->setSRID(53);
-
-        $writer->setOutputDimension(2); // 2D
-
-        // 2D LITTLE endian (3D input)
-        $writer->setByteOrder(1);
-        $this->assertEquals('010100000000000000000018400000000000001C40',
-            $writer->writeHEX($g));
-        // 2D LITTLE endian + SRID (3D input)
-        $writer->setIncludeSRID(TRUE);
-        $writer->setByteOrder(1);
-        $this->assertEquals(
-            '01010000203500000000000000000018400000000000001C40',
-            $writer->writeHEX($g));
-        $writer->setIncludeSRID(FALSE);
-        // 2D BIG endian (3D input)
-        $writer->setByteOrder(0);
-        $this->assertEquals('00000000014018000000000000401C000000000000',
-            $writer->writeHEX($g));
-        // 2D BIG endian + SRID (3D input)
-        $writer->setIncludeSRID(TRUE);
-        $this->assertEquals(
-            '0020000001000000354018000000000000401C000000000000',
-            $writer->writeHEX($g));
-        $writer->setIncludeSRID(FALSE);
-
-        $writer->setOutputDimension(3); // 3D
-
-        // 3D LITTLE endian (3D input)
-        $writer->setByteOrder(1);
-        $this->assertEquals(
-            '010100008000000000000018400000000000001C400000000000002040',
-            $writer->writeHEX($g));
-        // 3D BIG endian (3D input)
-        $writer->setByteOrder(0); 
-        $this->assertEquals(
-            '00800000014018000000000000401C0000000000004020000000000000',
-            $writer->writeHEX($g));
-        // 3D BIG endian + SRID (3D input)
-        $writer->setIncludeSRID(TRUE);
-        $this->assertEquals(
-          '00A0000001000000354018000000000000401C0000000000004020000000000000',
-          $writer->writeHEX($g));
-        $writer->setIncludeSRID(FALSE);
+    /**
+     * @expectedException \Exception
+     * @expectedExceptionMessage expects parameter 1
+     */
+    public function testInvalidWriteHEXThrowsException()
+    {
+        $writer = new GEOSWKBWriter();
+        $writer->writeHEX(1);
     }
 
     public function testWKBReader__construct()
@@ -2225,7 +2328,14 @@ MULTIPOINT(
         $this->assertNotNull($reader);
     }
 
-    public function testWKBReader_readHEX()
+    /**
+     * @dataProvider providerWKBReader_read
+     *
+     * @param string  $wkb  The WKB to read.
+     * @param boolean $is3D Whether the geometry is 3D (true) or 2D (false).
+     * @param integer $srid The expected result SRID.
+     */
+    public function testWKBReader_read($wkb, $is3D, $srid)
     {
         $reader = new GEOSWKBReader();
 
@@ -2233,56 +2343,29 @@ MULTIPOINT(
         $writer->setTrim(TRUE);
         $writer->setOutputDimension(3);
 
-        
-        // 2D LITTLE endian
-        $g = $reader->readHEX(
-            '010100000000000000000018400000000000001C40'
-        );
-        $this->assertEquals('POINT (6 7)', $writer->write($g));
-        $this->assertEquals(0, $g->getSRID());
+        $wkt = $is3D ? 'POINT Z (6 7 8)' : 'POINT (6 7)';
 
-        // 2D BIG endian
-        $g = $reader->readHEX(
-            '00000000014018000000000000401C000000000000'
-        );
-        $this->assertEquals('POINT (6 7)', $writer->write($g));
-        $this->assertEquals(0, $g->getSRID());
+        $g = $reader->read(hex2bin($wkb));
+        $this->assertSame($wkt, $writer->write($g));
+        $this->assertSame($srid, $g->getSRID());
 
-        // 2D LITTLE endian + SRID 
-        $g = $reader->readHEX(
-            '01010000202B00000000000000000018400000000000001C40'
-        );
-        $this->assertEquals('POINT (6 7)', $writer->write($g));
-        $this->assertEquals(43, $g->getSRID());
+        $g = $reader->readHEX($wkb);
+        $this->assertSame($wkt, $writer->write($g));
+        $this->assertSame($srid, $g->getSRID());
+    }
 
-        // 2D BIG endian + SRID
-        $g = $reader->readHEX(
-            '00200000010000002B4018000000000000401C000000000000'
+    public function providerWKBReader_read()
+    {
+        return array(
+            array('010100000000000000000018400000000000001C40',                         false,  0), // 2D LITTLE endian,
+            array('00000000014018000000000000401C000000000000',                         false,  0), // 2D BIG endian,
+            array('01010000202B00000000000000000018400000000000001C40',                 false, 43), // 2D LITTLE endian + SRID
+            array('00200000010000002B4018000000000000401C000000000000',                 false, 43), // 2D BIG endian + SRID,
+            array('010100008000000000000018400000000000001C400000000000002040',         true,   0), // 3D LITTLE endian
+            array('01010000A03500000000000000000018400000000000001C400000000000002040', true,  53), // 3D LITTLE endian + SRID
+            array('00800000014018000000000000401C0000000000004020000000000000',         true,   0), // 3D BIG endian
+            array('00A0000001000000354018000000000000401C0000000000004020000000000000', true,  53), // 3D BIG endian + SRID
         );
-        $this->assertEquals('POINT (6 7)', $writer->write($g));
-        $this->assertEquals(43, $g->getSRID());
-
-        // 3D LITTLE endian 
-        $g = $reader->readHEX(
-            '010100008000000000000018400000000000001C400000000000002040'
-        );
-        $this->assertEquals('POINT Z (6 7 8)', $writer->write($g));
-        $this->assertEquals(0, $g->getSRID());
-
-        // 3D BIG endian 
-        $g = $reader->readHEX(
-            '00800000014018000000000000401C0000000000004020000000000000'
-        );
-        $this->assertEquals('POINT Z (6 7 8)', $writer->write($g));
-        $this->assertEquals(0, $g->getSRID());
-
-        // 3D BIG endian + SRID
-        $g = $reader->readHEX(
-          '00A0000001000000354018000000000000401C0000000000004020000000000000'
-        );
-        $this->assertEquals('POINT Z (6 7 8)', $writer->write($g));
-        $this->assertEquals(53, $g->getSRID());
-
     }
 
     public function testGEOSRelateMatch()
@@ -2292,5 +2375,4 @@ MULTIPOINT(
         $this->assertTrue(GEOSRelateMatch('0FFFFFFF2', 'TFFFFFFF2'));
         $this->assertFalse(GEOSRelateMatch('0FFFFFFF2', '0FFFFFFFF'));
     }
-
 }
