@@ -1,8 +1,7 @@
 /**********************************************************************
- * $Id: Geometry.cpp 3634 2012-05-21 14:12:06Z strk $
  *
  * GEOS - Geometry Engine Open Source
- * http://geos.refractions.net
+ * http://geos.osgeo.org
  *
  * Copyright (C) 2009 2011 Sandro Santilli <strk@keybit.net>
  * Copyright (C) 2005-2006 Refractions Research Inc.
@@ -35,13 +34,13 @@
 #include <geos/geom/MultiPolygon.h>
 #include <geos/geom/IntersectionMatrix.h>
 #include <geos/util/IllegalArgumentException.h>
-#include <geos/algorithm/CentroidPoint.h>
-#include <geos/algorithm/CentroidLine.h>
-#include <geos/algorithm/CentroidArea.h>
+#include <geos/algorithm/Centroid.h>
 #include <geos/algorithm/InteriorPointPoint.h>
 #include <geos/algorithm/InteriorPointLine.h>
 #include <geos/algorithm/InteriorPointArea.h>
 #include <geos/algorithm/ConvexHull.h>
+#include <geos/operation/intersection/Rectangle.h>
+#include <geos/operation/intersection/RectangleIntersection.h>
 #include <geos/operation/predicate/RectangleContains.h>
 #include <geos/operation/predicate/RectangleIntersects.h>
 #include <geos/operation/relate/RelateOp.h>
@@ -63,7 +62,14 @@
 #include <cassert>
 #include <memory>
 
+#ifdef _MSC_VER
+#  ifdef MSVC_USE_VLD
+#    include <vld.h>
+#  endif
+#endif
+
 #define SHORTCIRCUIT_PREDICATES 1
+//#define USE_RECTANGLE_INTERSECTION 1
 
 using namespace std;
 using namespace geos::algorithm;
@@ -203,30 +209,8 @@ bool
 Geometry::getCentroid(Coordinate& ret) const
 {
 	if ( isEmpty() ) { return false; }
-
-	Coordinate c;
-
-	int dim=getDimension();
-	if(dim==0) {
-		CentroidPoint cent; 
-		cent.add(this);
-		if ( ! cent.getCentroid(c) )
-				return false;
-	} else if (dim==1) {
-		CentroidLine cent;
-		cent.add(this);
-		if ( ! cent.getCentroid(c) ) 
-			return false;
-	} else {
-		CentroidArea cent;
-		cent.add(this);
-		if ( ! cent.getCentroid(c) )
-			return false;
-	}
-
-	getPrecisionModel()->makePrecise(c);
-	ret=c;
-
+	if ( ! Centroid::getCentroid(*this, ret) ) return false;
+	getPrecisionModel()->makePrecise(ret); // not in JTS
 	return true;
 }
 
@@ -454,6 +438,10 @@ Geometry::equals(const Geometry *g) const
 	if (! getEnvelopeInternal()->equals(g->getEnvelopeInternal()))
 		return false;
 #endif
+
+	if (isEmpty()) return g->isEmpty();
+	else if (g->isEmpty()) return isEmpty();
+
 	auto_ptr<IntersectionMatrix> im ( relate(g) );
 	bool res=im->isEquals(getDimension(), g->getDimension());
 	return res;
@@ -515,8 +503,8 @@ Geometry*
 Geometry::intersection(const Geometry *other) const
 {
 	/**
-         * TODO: MD - add optimization for P-A case using Point-In-Polygon
-         */
+   * TODO: MD - add optimization for P-A case using Point-In-Polygon
+   */
 
 	// special case: if one input is empty ==> empty
 	if (isEmpty() || other->isEmpty() )
@@ -524,6 +512,24 @@ Geometry::intersection(const Geometry *other) const
 		return getFactory()->createGeometryCollection();
 	}
 
+#ifdef USE_RECTANGLE_INTERSECTION
+	// optimization for rectangle arguments
+  using operation::intersection::Rectangle;
+  using operation::intersection::RectangleIntersection;
+	if ( isRectangle() ) {
+    const Envelope* env = getEnvelopeInternal();
+    Rectangle rect(env->getMinX(), env->getMinY(),
+                   env->getMaxX(), env->getMaxY());
+    return RectangleIntersection::clip(*other, rect).release();
+  }
+	if (other->isRectangle()) {
+    const Envelope* env = other->getEnvelopeInternal();
+    Rectangle rect(env->getMinX(), env->getMinY(),
+                   env->getMaxX(), env->getMaxY());
+    return RectangleIntersection::clip(*this, rect).release();
+  }
+#endif
+
 	return BinaryOp(this, other, overlayOp(OverlayOp::opINTERSECTION)).release();
 }
 
@@ -706,6 +712,11 @@ Geometry::getClassSortIndex() const
 #endif
 }
 
+void Geometry::GeometryChangedFilter::filter_rw(Geometry* geom)
+{
+   geom->geometryChangedAction();
+}
+
 int
 Geometry::compare(vector<Coordinate> a, vector<Coordinate> b) const
 {
