@@ -1,8 +1,7 @@
 /**********************************************************************
- * $Id: PlanarGraph.cpp 3281 2011-04-18 15:08:24Z strk $
  *
  * GEOS - Geometry Engine Open Source
- * http://geos.refractions.net
+ * http://geos.osgeo.org
  *
  * Copyright (C) 2005-2006 Refractions Research Inc.
  * Copyright (C) 2001-2002 Vivid Solutions Inc.
@@ -14,7 +13,7 @@
  *
  **********************************************************************
  *
- * Last port: geomgraph/PlanarGraph.java rev. 1.6 (JTS-1.10)
+ * Last port: geomgraph/PlanarGraph.java r428 (JTS-1.12+)
  *
  **********************************************************************/
 
@@ -72,6 +71,10 @@ PlanarGraph::PlanarGraph()
 /*public*/
 PlanarGraph::~PlanarGraph()
 {
+#if GEOS_DEBUG
+	std::cerr << "~PlanarGraph" << std::endl;
+#endif
+
 	delete nodes;
 #if 1 // FIXME: PlanarGraph should *not* own edges!
 	for(size_t i=0, n=edges->size(); i<n; i++) {
@@ -110,8 +113,8 @@ PlanarGraph::isBoundaryNode(int geomIndex, const Coordinate& coord)
 	Node *node=nodes->find(coord);
 	if (node==NULL) return false;
 
-	Label *label=node->getLabel();
-	if (label!=NULL && label->getLocation(geomIndex)==Location::BOUNDARY)
+	const Label& label = node->getLabel();
+	if (! label.isNull() && label.getLocation(geomIndex)==Location::BOUNDARY)
 			return true;
 
 	return false;
@@ -167,7 +170,7 @@ Node*
 PlanarGraph::addNode(Node *node)
 {
 	assert(nodes);
-#if GEOS_DEBUG
+#if GEOS_DEBUG > 1
 	cerr << "PlanarGraph::addNode(Node * " << *node 
 		<< ")" << endl;
 #endif
@@ -178,7 +181,7 @@ PlanarGraph::addNode(Node *node)
 Node*
 PlanarGraph::addNode(const Coordinate& coord)
 {
-#if GEOS_DEBUG
+#if GEOS_DEBUG > 1
 	cerr << "PlanarGraph::addNode(Coordinate& "
 		<< coord << ")" << endl;
 #endif
@@ -377,40 +380,3 @@ PlanarGraph::getNodeMap()
 } // namespace geos.geomgraph
 } // namespace geos
 
-/**********************************************************************
- * $Log$
- * Revision 1.33  2006/06/12 11:29:23  strk
- * unsigned int => size_t
- *
- * Revision 1.32  2006/05/04 12:17:27  strk
- * Added comment about management of newly created DirectedEdges
- *
- * Revision 1.31  2006/04/27 15:07:15  strk
- * use output operators in debugging lines
- *
- * Revision 1.30  2006/04/07 16:52:20  strk
- * Port info, doxygen comments, assertion checking
- *
- * Revision 1.29  2006/03/15 17:16:29  strk
- * streamlined headers inclusion
- *
- * Revision 1.28  2006/03/14 15:46:53  strk
- * Added PlanarGraph::getNodes(vector&) func, to reduce useless heap allocations
- *
- * Revision 1.27  2006/03/09 16:46:47  strk
- * geos::geom namespace definition, first pass at headers split
- *
- * Revision 1.26  2006/03/06 19:40:46  strk
- * geos::util namespace. New GeometryCollection::iterator interface, many cleanups.
- *
- * Revision 1.25  2006/03/03 10:46:21  strk
- * Removed 'using namespace' from headers, added missing headers in .cpp files, removed useless includes in headers (bug#46)
- *
- * Revision 1.24  2006/03/02 16:21:26  strk
- * geos::precision namespace added
- *
- * Revision 1.23  2006/03/02 12:12:00  strk
- * Renamed DEBUG macros to GEOS_DEBUG, all wrapped in #ifndef block to allow global override (bug#43)
- *
- **********************************************************************/
-
