@@ -1,4 +1,3 @@
-// $Id: PointTest.cpp 3180 2011-02-03 21:50:17Z strk $
 // 
 // Test Suite for geos::geom::Point class.
 
@@ -372,7 +371,7 @@ namespace tut
 	{
 		GeometryAutoPtr geo(empty_point_->clone());
 
-		ensure( !empty_point_->equals(geo.get()) );
+		ensure( empty_point_->equals(geo.get()) );
 	}
 
 	// Test of equals() for non-empty Point (1.234,5.678)
